DROP TABLE fingerprint CASCADE CONSTRAINTS ; 
DROP TABLE filetype CASCADE CONSTRAINTS ;
drop sequence filetype_seq
DROP TABLE filename CASCADE CONSTRAINTS ; 
drop sequence filename_seq

-- filename table
create table filename (
  filenameid integer not null,
  filename varchar2(500),
  CONSTRAINT CNN_000001 CHECK ( filenameid IS NOT NULL ), 
  CONSTRAINT CNN_000002 CHECK ( filename IS NOT NULL ),
  CONSTRAINT UNQ_filename UNIQUE (filename),
  CONSTRAINT PK_filename PRIMARY KEY ( filenameid )
);
create sequence filename_seq start with 1 increment by 1;
create or replace trigger filename_autonumber
before insert on filename for each row
begin
    if :new.filenameid is null then
        select filename_seq.nextval into :new.filenameid from dual;
    end if;
end;
/
-- filetype table
create table filetype (
  filetypeid integer not null,
  filetype varchar2(100),
  CONSTRAINT CNN_000003 CHECK ( filetypeid IS NOT NULL ), 
  CONSTRAINT CNN_000004 CHECK ( filetype IS NOT NULL ),
  CONSTRAINT UNQ_filetype UNIQUE (filetype),
  CONSTRAINT PK_filetype PRIMARY KEY ( filetypeid )
);
create sequence filetype_seq start with 1 increment by 1;
create or replace trigger filetype_autonumber
before insert on filetype for each row
begin
    if :new.filetypeid is null then
        select filetype_seq.nextval into :new.filetypeid from dual;
    end if;
end;
/
-- fingerprint table
create table fingerprint (
  filenameid integer,
  filetypeid integer,
  fingerprint varchar2(2000)
);
ALTER TABLE fingerprint ADD CONSTRAINT FK_fingerprint_nameid
FOREIGN KEY (filenameid) 
REFERENCES filename (filenameid) ;
ALTER TABLE fingerprint ADD CONSTRAINT FK_fingerprint_typeid
FOREIGN KEY (filetypeid) 
REFERENCES filetype (filetypeid) ;

insert into status values (-1,'unknown');
insert into status values (0,'OK');
insert into status values (1,'too short');
insert into status values (2,'signature mismatch');
insert into status values (3,'not readable');
insert into status values (4,'invalid');
insert into status values (5,'corrupt');
insert into status values (6,'empty');
insert into status values (7,'missing');

commit;
