package net.za.grasser.duplicate.file;

import java.io.Serializable;

/**
 * This enum defines the statuses that a FingerprintFile can have.
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 */
public enum Status implements Serializable {
  /**
   * <code>FILE_MISSING</code> Status -
   */
  FILE_MISSING(7, "FILE_MISSING"),
  /**
   * <code>FILE_EMPTY</code> Status -
   */
  FILE_EMPTY(6, "FILE_EMPTY"),
  /**
   * <code>FILE_CORRUPT</code> Status -
   */
  FILE_CORRUPT(5, "FILE_CORRUPT"),
  /**
   * <code>FILE_INVALID</code> Status -
   */
  FILE_INVALID(4, "FILE_INVALID"),
  /**
   * <code>FILE_NOT_READABLE</code> Status -
   */
  FILE_NOT_READABLE(3, "FILE_NOT_READABLE"),
  /**
   * <code>FILE_SIGNATURE_MISMATCH</code> Status -
   */
  FILE_SIGNATURE_MISMATCH(2, "FILE_SIGNATURE_MISMATCH"),
  /**
   * <code>FILE_TOO_SHORT</code> Status -
   */
  FILE_TOO_SHORT(1, "FILE_TOO_SHORT"),
  /**
   * <code>FILE_OK</code> Status -
   */
  FILE_OK(0, "FILE_OK"),
  /**
   * <code>FILE_UNKNOWN</code> Status -
   */
  FILE_UNKNOWN(-1, "FILE_UNKNOWN");
  /**
   * <code>value</code> Status - internal value (also used in the db)
   */
  private final int value;
  /**
   * <code>description</code> Status -
   */
  private final String description;

  /**
   * Sole constructor.
   * 
   * @param pValue
   * @param pDesc
   */
  private Status(final int pValue, final String pDesc) {
    value = pValue;
    description = pDesc;
  }

  /**
   * This method ...
   * 
   * @param pVal
   * @return Status
   */
  public static Status find(final int pVal) {
    for (final Status lStat : values()) {
      if (lStat.value == pVal) {
        return lStat;
      }
    }
    return FILE_UNKNOWN;
  }

  /**
   * @return the value
   */
  public int getValue() {
    return value;
  }

  /**
   * @return the description
   */
  public String getDescription() {
    return description;
  }

  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return description + " (" + value + ")";
  }
}