/**
 * DuMP3 version morpheus_0.2.9 - a duplicate/similar file finder in Java<BR>
 * Copyright 2005 Alexander Gr&auml;sser<BR>
 * All Rights Reserved, http://dump3.sourceforge.net/<BR>
 * <BR>
 * This file is part of DuMP3.<BR>
 * <BR>
 * DuMP3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later version.<BR>
 * <BR>
 * DuMP3 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.<BR>
 * <BR>
 * You should have received a copy of the GNU General Public License along with DuMP3; if not, write to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.za.grasser.duplicate.fingerprint;

import java.awt.Point;
import java.io.File;
import java.io.InputStream;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.AudioFormat.Encoding;
import javax.sound.sampled.spi.FormatConversionProvider;
import net.za.grasser.duplicate.Configure;
import net.za.grasser.duplicate.file.FingerprintFile;
import net.za.grasser.duplicate.file.Status;
import net.za.grasser.duplicate.fingerprint.configure.ConfigFactory;
import net.za.grasser.duplicate.fingerprint.configure.FdmfFingerprintConfig;
import net.za.grasser.duplicate.util.fft.Complex;
import net.za.grasser.duplicate.util.fft.FFT;
import org.apache.log4j.Logger;
import com.sun.media.sound.JDK13Services;

/**
 * REF: <a href="http://www.w140.com/audio/">fdmf</a>Thomas Jarosch, Aug 2004
 * 
 * @author <a href="http://sourceforge.net/sendmessage.php?touser=733840">pyropunk at sourceforge dot net</a>
 * @version $Revision: 1.15 $
 * @modelguid {87AC7397-9A44-4C1C-B351-255DFA8ACEA3}
 */
public class FdmfFingerprint extends AbstractFingerprint {
  /**
   * <code>log</code> FdmfFingerprint -
   */
  private static final Logger log = Logger.getLogger(FdmfFingerprint.class);
  static {
    Configure.load();
    final Set<String> set = new HashSet<String>();
    final Iterator<FormatConversionProvider> it = JDK13Services.getProviders(FormatConversionProvider.class).iterator();
    while (it.hasNext()) {
      final FormatConversionProvider fcp = it.next();
      final Encoding[] as = fcp.getSourceEncodings();
      for (final Encoding element : as) {
        set.add(element.toString());
      }
    }
    if (log.isInfoEnabled()) {
      final StringBuffer s = new StringBuffer(100);
      s.append("[");
      final Iterator<String> it2 = set.iterator();
      if (it2.hasNext()) {
        s.append(it2.next());
      }
      while (it2.hasNext()) {
        s.append(", ").append(it2.next());
      }
      s.append(']');
      log.info("Supported file formats:" + s);
    }
  }
  /**
   * <code>WINDOW_RECTANGULAR</code> FdmfFingerprint -
   */
  public static final int WINDOW_RECTANGULAR = 0;
  /**
   * <code>WINDOW_HANNING</code> FdmfFingerprint -
   */
  public static final int WINDOW_HANNING = 1;
  /**
   * <code>WINDOW_HAMMING</code> FdmfFingerprint -
   */
  public static final int WINDOW_HAMMING = 2;
  /**
   * <code>WINDOW_RAISED_COSINE</code> FdmfFingerprint -
   */
  public static final int WINDOW_RAISED_COSINE = 3;
  /**
   * <code>WINDOW_WELCH</code> FdmfFingerprint -
   */
  public static final int WINDOW_WELCH = 4;
  /**
   * <code>WINDOW_BARTLETT</code> FdmfFingerprint -
   */
  public static final int WINDOW_BARTLETT = 5;
  /**
   * <code>WINDOW_KAISER</code> FdmfFingerprint -
   */
  public static final int WINDOW_KAISER = 6;
  /**
   * <code>CHUNK_SAMPLES</code> FdmfFingerprint - approx. 375 milliseconds
   */
  public static final int CHUNK_SAMPLES = 16384;
  /**
   * <code>MAXCHUNKS</code> FdmfFingerprint - (65536 = 6.7 hours) (256 = 1.58 min) at 371 ms per chunk
   */
  private static final int MAX_CHUNKS = 256;
  /**
   * <code>NUM_BANDS</code> FdmfFingerprint - number of frequency bands to analyse
   */
  private static final int NUM_BANDS = 4;
  /**
   * <code>frequency</code> FdmfFingerprint -
   */
  private static final int[] frequency = {
      3, 15, 90, 600, 5000
  };
  /**
   * <code>config</code> FdmfFingerprint -
   */
  private static final FdmfFingerprintConfig config = (FdmfFingerprintConfig)ConfigFactory.getConfig(FdmfFingerprint.class);

  /**
   * compare the bit counts of two byte arrays
   * 
   * @param a
   * @param b
   * @return percentage of matching bits
   */
  private static double compare(final byte[] a, final byte[] b) {
    final double lena = a.length / 3;
    final double lenb = b.length / 3;
    final double len = Math.min(lena, lenb);
    double match = 0.0;
    for (int j = 0; j < 3; j++) {
      for (int i = 0; i < len; i++) {
        final BigInteger x = new BigInteger(("" + (a[(j * (int)lena + i)] & 0xff ^ b[(j * (int)lenb + i)] & 0xff ^ 0xff)));
        match += x.bitCount();
      }
    }
    return match / (len * 24.0) * 100.0;
  }

  /**
   * @param pList
   * @return median fo the list
   */
  private static double median(final List<Double> pList) {
    final int length = pList.size();
    final List<Double> clone = new ArrayList<Double>(pList);
    Collections.sort(clone);
    if (length % 2 == 0) {
      // even - median = average of two middle values
      return (clone.get((length >> 1)).doubleValue() + clone.get((length - 2 >> 1)).doubleValue()) / 2.0;
    }
    // odd - median = middle value
    return clone.get((length - 1 >> 1)).doubleValue();
  }

  /**
   * one bit quantize with median value as threshold
   * 
   * @param pArray
   * @return quantized byte array
   */
  private static byte[] quantize(final Complex[] pArray) {
    final List<Double> list = new ArrayList<Double>(pArray.length);
    for (final Complex element : pArray) {
      list.add(new Double(element.abs()));
    }
    final double median = median(list);
    final byte[] ba = new byte[list.size() >> 3];
    final Iterator<Double> it = list.iterator();
    int i = 7;
    int j = 0;
    ba[0] = 0;
    while (it.hasNext()) {
      final double d = it.next().doubleValue();
      if (d > median) {
        ba[j] |= 1 << i;
      }
      if (i <= 0) {
        j++;
        i = 7;
        if (it.hasNext()) {
          ba[j] = 0;
        }
      } else {
        i--;
      }
    }
    return ba;
  }

  /**
   * <code>started</code> FdmfFingerprint - used to ignore starting silence
   */
  boolean started = true;
  /**
   * <code>fftPos</code> FdmfFingerprint - position in fft array
   */
  int fftPos = 0;
  /**
   * <code>fftInputArray</code> FdmfFingerprint - array of input values for the FFT
   */
  Complex[] fftInputArray = null;
  /**
   * <code>chunkPos</code> FdmfFingerprint -
   */
  int chunkPos = 0;
  /**
   * <code>energyBuffer</code> FdmfFingerprint -
   */
  Complex[] energyBuffer = null;
  /**
   * <code>ratioBuffer</code> FdmfFingerprint -
   */
  Complex[] ratioBuffer = null;
  /**
   * <code>twistBuffer</code> FdmfFingerprint -
   */
  Complex[] twistBuffer = null;

  /**
   * @param fi FingerprintFile
   * @modelguid {53F63DBA-703A-4B81-882B-F1B4318D5CC9}
   */
  public FdmfFingerprint(final FingerprintFile fi) {
    super(fi);
  }

  /**
   * This method adds additional info to the FingerprintFile
   */
  @Override
  public void addInfo() {
    super.addInfo();
    try {
      final Map<String, String> map = getFile().getInfo();
      final File file = new File(getFile().getPath());
      // AudioFileFormat properties
      final AudioFileFormat baseFileFormat = AudioSystem.getAudioFileFormat(file);
      map.put("format", AudioSystem.getAudioInputStream(file).getFormat().toString());
      Map<String, Object> properties = baseFileFormat.properties();
      if (properties != null) {
        for (final String lString : properties.keySet()) {
          // read InputStreams
          final Object o = properties.get(lString);
          if (o instanceof InputStream) {
            final InputStream tag = (InputStream)o;
            final byte[] buff = new byte[256];
            final StringBuffer sb = new StringBuffer();
            int len = tag.read(buff);
            while (len > 0) {
              sb.append(new String(buff, 0, len));
              len = tag.read(buff);
            }
          } else {
            // otherwise convert to String
            map.put(lString, o.toString());
          }
        }
      }
      // AudioFormat properties
      properties = baseFileFormat.getFormat().properties();
      if (properties != null) {
        for (final String lString : properties.keySet()) {
          map.put(lString, properties.get(lString).toString());
        }
      }
    } catch (final Throwable t) {
      log.error("Could not get info", t);
    }
  }

  /**
   * This routine takes in a data structure that is the output of an FFT operation. The input to this routine is in the frequency domain. The power is
   * integrated over NUM_BANDS non-overrlapping frequency bands. These band energies are returned to the caller in an array of double.
   * 
   * @param fftOutputArray Complex array output by FFT
   * @return array of band energies
   */
  private double[] calculateBandEnergies(final Complex[] fftOutputArray) {
    final double[] be = new double[NUM_BANDS];
    int b = 0;
    for (b = 0; b < NUM_BANDS; b++) {
      int i = 0;
      be[b] = 0.0;
      for (i = frequency[b]; i < frequency[b + 1]; i++) {
        be[b] += fftOutputArray[i].scale();
      }
      be[b] /= CHUNK_SAMPLES;
      be[b] = Math.sqrt(be[b]);
    }
    return be;
  }

  /**
   * This routine calculates some metrics on the band energies of a chunk. The metrics are written to arrays, with one array for each metric. The chunk argument
   * is the position in the output arrays where the metrics should be stored.
   * 
   * @param be band energies
   */
  void chunkMetrics(final double[] be) {
    final double energy;
    double ratio = 0.0;
    double twist = 0.0;
    double lows = 0.0;
    final double highs;
    final double evens;
    double odds = 0.0;
    lows = be[0] + be[1];
    highs = be[2] + be[3];
    evens = be[0] + be[2];
    odds = be[1] + be[3];
    /* trap zeros */
    lows = Math.abs(lows) < 0.001 ? 0.001 : lows;
    odds = Math.abs(odds) < 0.001 ? 0.001 : odds;
    energy = lows + highs;
    ratio = highs / lows;
    twist = evens / odds;
    ratio = Math.abs(ratio) > 20.0 ? 20 : ratio;
    twist = Math.abs(twist) > 20.0 ? 20 : twist;
    energyBuffer[chunkPos] = new Complex(energy, 0.0);
    ratioBuffer[chunkPos] = new Complex(ratio, 0.0);
    twistBuffer[chunkPos] = new Complex(twist, 0.0);
  }

  /**
   * This routine takes raw audio data and puts it into the data structure that FFT operates on.
   * 
   * @param buffer a byte array
   * @param in a Complex array that id passed to the FFT
   * @param offs offset in the byte array. i.e. starting position
   * @param length length of the byte array
   * @param pos offset in the Complex array
   * @return a Point with the positions of the fft Array and the byte array
   */
  private Point convertAudioToComplex(final byte[] buffer, final Complex[] in, final int offs, final int length, final int pos) {
    int j = offs;
    int i = pos;
    for (; i < CHUNK_SAMPLES && j < length; i++) {
      int l = 0;
      int r = 0;
      l = buffer[j++] & 0xff;
      l |= buffer[j++] << 8 & 0xff00;
      r = buffer[j++] & 0xff;
      r |= buffer[j++] << 8 & 0xff00;
      in[i] = new Complex(l + r, 0.0);
    }
    return new Point(i, j);
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.AbstractFingerprint#getClassName()
   */
  @Override
  public String getClassName() {
    final StringBuffer sb = new StringBuffer();
    sb.append(super.getClassName()).append("-");
    switch (config.getWindowType()) {
      default:
      case WINDOW_RECTANGULAR:
        sb.append("RECT");
        break;
      case WINDOW_HAMMING:
        sb.append("HAMM");
        break;
      case WINDOW_HANNING:
        sb.append("HANN");
        break;
      case WINDOW_RAISED_COSINE:
        sb.append("RACO-").append(config.getAlpha());
        break;
      case WINDOW_BARTLETT:
        sb.append("BART");
        break;
      case WINDOW_WELCH:
        sb.append("WELC");
        break;
      case WINDOW_KAISER:
        sb.append("KAIS-").append(config.getAlpha());
        break;
    }
    return sb.toString();
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.AbstractFingerprint#getSimilarityThreshhold()
   */
  @Override
  public float getSimilarityThreshhold() {
    return config.getSimilarityThreshhold();
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.AbstractFingerprint#makeFingerprintFromFile()
   */
  @Override
  protected void makeFingerprintFromFile() {
    final File f = new File(getFile().getPath());
    AudioFormat audioFormat = new AudioFormat(44100.0f, 16, 2, true, false);
    try {
      AudioInputStream convertedAIS = null;
      final AudioInputStream inFileAIS = AudioSystem.getAudioInputStream(f);
      if (audioFormat.equals(inFileAIS.getFormat())) {
        convertedAIS = inFileAIS;
      } else if (inFileAIS.getFormat().getEncoding() == AudioFormat.Encoding.PCM_SIGNED
          || inFileAIS.getFormat().getEncoding() == AudioFormat.Encoding.PCM_UNSIGNED) {
        // hack for WAV files
        audioFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 44100.0f, 16, 2, inFileAIS.getFormat().getFrameSize(), 44100.0f, false);
        convertedAIS = AudioSystem.getAudioInputStream(audioFormat, inFileAIS);
      } else if (AudioSystem.isConversionSupported(audioFormat, inFileAIS.getFormat())) {
        convertedAIS = AudioSystem.getAudioInputStream(audioFormat, inFileAIS);
      } else {
        throw new Exception("Could not convert from [" + inFileAIS.getFormat().toString() + "] to [" + audioFormat.toString() + "]");
      }
      final DataLine.Info info = new DataLine.Info(SourceDataLine.class, audioFormat);
      if (!AudioSystem.isLineSupported(info)) {
        throw new Exception("audio type not supported.");
      }
      final SourceDataLine dataLine = (SourceDataLine)AudioSystem.getLine(info);
      dataLine.open(audioFormat);
      dataLine.start();
      preRead();
      final int bufferSize = (int)audioFormat.getSampleRate() * audioFormat.getFrameSize();
      byte[] buffer = new byte[bufferSize];
      int bytesRead = 0;
      while ((bytesRead = convertedAIS.read(buffer, 0, bufferSize)) >= 0 && chunkPos < MAX_CHUNKS) {
        update(buffer, bytesRead);
      }
      // release memory as soon as possible
      buffer = null;
      dataLine.drain();
      dataLine.close();
      postRead();
    } catch (final Exception e) {
      getFile().setStatus(Status.FILE_CORRUPT);
      fingerprint = null;
      log.warn("error while reading file [" + getFile().getPath() + "]", e);
    }
  }

  /**
   * @param fi AbstractFingerprint
   * @return boolean
   * @modelguid {3E6D5504-3A75-4444-BE5B-C23BB0DF4DE8}
   */
  @Override
  public float matches(final AbstractFingerprint fi) {
    final FingerprintFile ff1 = getFile();
    final FingerprintFile ff2 = fi.getFile();
    if (ff1.getStatus() != Status.FILE_OK && ff1.getStatus() != Status.FILE_SIGNATURE_MISMATCH || ff2.getStatus() != Status.FILE_OK
        && ff2.getStatus() != Status.FILE_SIGNATURE_MISMATCH) {
      return 0.0f;
    }
    if (ff1.getLength() == 0L && ff2.getLength() == 0L) {
      return 100.0f;
    }
    if (fi instanceof FdmfFingerprint) {
      final FdmfFingerprint pf = (FdmfFingerprint)fi;
      // try to read the files
      final byte[] a = getFingerprint();
      final byte[] b = pf.getFingerprint();
      if ((ff1.getStatus() == Status.FILE_OK || ff1.getStatus() == Status.FILE_SIGNATURE_MISMATCH)
          && (ff2.getStatus() == Status.FILE_OK || ff2.getStatus() == Status.FILE_SIGNATURE_MISMATCH)) {
        if (a == null) {
          log.error("File [" + ff1.toString() + "] read Ok, but fingerprint is null!");
          ff1.setStatus(Status.FILE_CORRUPT);
          return 0.0f;
        }
        if (b == null) {
          log.error("File [" + ff2.toString() + "] read Ok, but fingerprint is null!");
          ff2.setStatus(Status.FILE_CORRUPT);
          return 0.0f;
        }
        final float c = (float)compare(a, b);
        return c;
      }
    }
    return 0.0f;
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.AbstractFingerprint#postRead()
   */
  @Override
  protected void postRead() {
    // calc fingerprint
    fftInputArray = null;
    if (chunkPos > 0) {
      // cut the buffers to a power of 2
      final int len = (int)Math.pow(2.0, ((int)(Math.log(chunkPos) / Math.log(2.0))));
      Complex[] eb = new Complex[len];
      Complex[] rb = new Complex[len];
      Complex[] tb = new Complex[len];
      System.arraycopy(energyBuffer, 0, eb, 0, len);
      System.arraycopy(ratioBuffer, 0, rb, 0, len);
      System.arraycopy(twistBuffer, 0, tb, 0, len);
      // release memory
      energyBuffer = null;
      ratioBuffer = null;
      twistBuffer = null;
      final double[] wt = config.makeWindow(len);
      // do fft
      eb = FFT.fft2(FFT.window(eb, wt));
      rb = FFT.fft2(FFT.window(rb, wt));
      tb = FFT.fft2(FFT.window(tb, wt));
      fingerprint = new byte[(eb.length + rb.length + tb.length >> 3)];
      final byte[] l1 = quantize(eb);
      System.arraycopy(l1, 0, fingerprint, 0, l1.length);
      final byte[] l2 = quantize(rb);
      System.arraycopy(l2, 0, fingerprint, l1.length, l2.length);
      final byte[] l3 = quantize(tb);
      System.arraycopy(l3, 0, fingerprint, l1.length + l2.length, l3.length);
    }
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.AbstractFingerprint#preRead()
   */
  @Override
  protected void preRead() {
    started = true;
    fftPos = 0;
    fftInputArray = new Complex[CHUNK_SAMPLES];
    chunkPos = 0;
    energyBuffer = new Complex[MAX_CHUNKS];
    ratioBuffer = new Complex[MAX_CHUNKS];
    twistBuffer = new Complex[MAX_CHUNKS];
  }

  /**
   * @see net.za.grasser.duplicate.fingerprint.AbstractFingerprint#update(byte[], int)
   */
  @Override
  protected void update(final byte[] pBuffer, final int pLength) {
    int baPos = 0;
    // find start of sound (ignore silence)
    if (started) {
      int i = 0;
      for (; i < pLength; i++) {
        if (pBuffer[i] != 0) {
          baPos = i;
          break;
        }
      }
      if (i == pLength) {
        return;
      }
    }
    started = false;
    Point ret = convertAudioToComplex(pBuffer, fftInputArray, baPos, pLength, fftPos);
    while (baPos < pLength && chunkPos < MAX_CHUNKS) {
      fftPos = ret.x;
      baPos = ret.y;
      if (fftPos == CHUNK_SAMPLES) {
        chunkMetrics(calculateBandEnergies(FFT.fft2(FFT.window(fftInputArray, config.getWindow()))));
        chunkPos++;
        // don't forget the last couple of bytes
        if (baPos < pLength) {
          ret = convertAudioToComplex(pBuffer, fftInputArray, baPos, pLength, 0);
        }
      }
    }
  }
}
